
# $rcs = ' $Id: Backends.pm,v 1.4 2006/07/30 22:35:00 Daddy Exp $ ' ;

=head1 NAME

WWW::Search::Backends

=head1 DESCRIPTION

This is an empty package.  It only exists so that CPAN users can say
"install WWW::Search::Backends" and they'll get what they expect.
Well, actually I'm not sure what they expect, but they'll get
something reasonable.

It is also used to control the version number of the distribution.

=head1 AUTHOR

Martin Thurn

=cut

package WWW::Search::Backends;

my
$VERSION = '1.04';

1;

__END__

